@@ -25,18 +25,16 @@ class PerfettoService(Service):
 	def config(self, **kwargs):
 		pass
 
-	def init(self,boot_time=0,**kwargs):
+	def init(self,boot_time=0, **kwargs):
 		self.boot_time = boot_time
 		self.clean()
 
 	def start(self):
-
 		execute_shell_command(f"adb shell perfetto -o {self.output_filename} freq  -t 1h --background ")
 	
-	def stop(self, file_id=None):
+	def stop(self, file_id):
 		if file_id is None:
 			file_id = execute_shell_command("date +%s")[1].strip()
-		#executeShCommand("adb shell su -c \"killall perfetto\"")
 		execute_shell_command("adb shell killall perfetto")
 		time.sleep(1)
 		execute_shell_command(f"adb pull {self.output_filename} " + ("%strace" % self.results_dir) + "-" + file_id + "-" + str(self.boot_time))
@@ -44,9 +42,9 @@ class PerfettoService(Service):
 
 	def export(self):
 		last_exported = ""
-		for f in filter( lambda x : re.match(r'trace-*', x) is not None,  os.listdir(self.results_dir)):
+		for f in filter(lambda x: re.match(r'trace-*', x) is not None,  os.listdir(self.results_dir)):
 			res, o, e = execute_shell_command("chmod +x %s/traceconv ; %s/traceconv systrace %s/%s %s/%s.systrace" %(RESOURCES_DIR,RESOURCES_DIR,self.results_dir,f,self.results_dir,f) )
-			last_exported = "%s/%s.systrace" %(self.results_dir,f)
+			last_exported = "%s/%s.systrace" %(self.results_dir, f)
 		return last_exported
 
 	def clean(self):
